/*******************************************************************************
*    Robot "Giskard" 1.0.                                                      *
*    Copyright (C) 2009 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    Circuits  by Marian Neubert     marian@tesla-crew.de                      *
*    Mechanism by Mario Zimmermann   mario@tesla-crew.de                       *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include <math.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "inc/giskard.h"
#include "inc/leds.h"
#include "inc/tools.h"

/**
* So viele IRQs werden jede Sekunde ausgelöst.
* Für optimale Genauigkeit muss
* IRQS_PER_SECOND ein Teiler von F_CPU sein
* und IRQS_PER_SECOND ein Vielfaches von 100.
* Ausserdem muss gelten F_CPU / IRQS_PER_SECOND <= 65536
*/
#define IRQS_PER_SECOND   1000   /* 1 ms */

/**
* Anzahl IRQs pro 10 Millisekunden
*/
#define IRQS_PER_10MS     (IRQS_PER_SECOND / 100)

/**
* Gültigkeitsprüfung.
* Bei ungeeigneten Werten gibt es einen Compilerfehler
*/
#if (F_CPU/IRQS_PER_SECOND > 65536) || (IRQS_PER_10MS < 1) || (IRQS_PER_10MS > 255)
#   error Diese Werte für F_CPU und IRQS_PER_SECOND
#   error sind ausserhalb des gueltigen Bereichs!
#endif

/**
* Compiler-Warnung falls die Genauigkeit nicht optimal ist.
* Wenn das nervt für deine Werte, einfach löschen :-)
*/
#if (F_CPU % IRQS_PER_SECOND != 0) || (IRQS_PER_SECOND % 100 != 0)
#   warning Das Programm arbeitet nicht mit optimaler Genauigkeit.
#endif

#if !defined (TCNT1H)
#error Dieser Controller hat keinen 16-Bit Timer1!
#endif // TCNT1H


/**
* Wurde in comm.c definiert
*/
extern boolean commInitialized;

/*************************************************************************
 delay loop for small accurate delays: 16-bit counter, 4 cycles/loop
*************************************************************************/
inline void _delayFourCycles(unsigned int __count)
{
  if ( __count == 0 )
    __asm__ __volatile__( "rjmp 1f\n 1:" );       // 2 cycles
  else
    __asm__ __volatile__ (
                          "1: sbiw %0,1" "\n\t"
                          "brne 1b"               // 4 cycles/loop
                          : "=w" (__count)
                          : "0" (__count)
    );
}

/**
* Die Interrupt Service Routine (ISR).
* Die Variabel "uptime" wird erhöht
*/
SIGNAL (SIG_OUTPUT_COMPARE1A)
{
  uptime++;
}

/**
* Initialisiert den Timer
*/
void doInitTimer()
{
  uptime = 0L;

  // Timer1: keine PWM
  TCCR1A = 0;

  // Timer1 ist Zähler: Clear Timer on Compare Match (CTC, Mode #4)
  // Timer1 läuft mit vollem MCU-Takt: Prescale = 1
#if defined (CTC1) && !defined (WGM12)
  TCCR1B = (1 << CTC1)  | (1 << CS10);
#elif !defined (CTC1) && defined (WGM12)
  TCCR1B = (1 << WGM12) | (1 << CS10);
#else
#error Keine Ahnung, wie Timer1 für diesen AVR zu initialisieren ist!
#endif

  // OutputCompare für gewünschte Timer1 Frequenz
  // TCNT1 zählt immer 0...OCR1A, 0...OCR1A, ...
  // Beim überlauf OCR1A -> OCR1A+1 wird TCNT1=0 gesetzt und im nächsten
  // MCU-Takt eine IRQ erzeugt.
  OCR1A = (unsigned short) ((unsigned long) F_CPU / IRQS_PER_SECOND-1);

  // OutputCompareA-Interrupt für Timer1 aktivieren
#if defined (TIMSK1)
  TIMSK1 |= (1 << OCIE1A);
#elif defined (TIMSK)
  TIMSK  |= (1 << OCIE1A);
#else
#error Keine Ahnung, wie IRQs für diesen AVR zu initialisieren sind!
#endif
}

/**
* Warte x Millisekunden
*
* @param uint8_t x   Die Millisekunden zu warten
*/
void delayMS(unsigned int ms)
{
  int l;

  for(l = 0; l < ms; l++)
    delayUS(1000);
}

/**
* Rundet die angegebene Zahl
*
* @param uint8_t x   Die zu rundenden Zahl
*/
double round(double x)
{
  double intPart;
  double decPart;

  intPart = floor(x);
  decPart = x - intPart;
  if(decPart > 0.5)
    return (intPart + 1);
  else
    return intPart;
}
